-------------------------------------------------------------------------------
--! @file       copa_datapath.vhd
--! @author     Farnoud Farahmand
--! @brief      Datapath unit for ASCON
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.all;


entity copa_datapath is
     port(
      bdi                   : in   std_logic_vector(127 downto 0);
		key                   : in   std_logic_vector(127 downto 0);
		bdo                   : out  std_logic_vector(127 downto 0);
		rst                   : in   std_logic;
		clk                   : in   std_logic;		  		  
		start1,start2         : in   std_logic;
		tag_match             : out  std_logic;
      ready1,ready2         : out  std_logic;
      done1,done2           : out  std_logic;
		done_init_both        : out  std_logic;
		bdi_valid_bytes : in STD_LOGIC_vector (15 downto 0);
      bdi_pad_loc	    : in STD_LOGIC_vector (15 downto 0);
		
-------MUX-INIT---------
      init_delta         : in   std_logic;
		init_pmac          : in   std_logic;
		init1,init2        : in   std_logic;
		init_Vi            : in   std_logic;
		init_Enc_Dec       : in   std_logic;
		init_Enc_Dec_tag   : in   std_logic;
		init_U             : in   std_logic;
		init_size_eq       : in   std_logic;
		init_in1           : in   std_logic_vector(2 downto 0);
		init_out           : in   std_logic;
		init_tag           : in   std_logic;  
		init_partial_delta  : in   std_logic;
--------EN-----------		  
		en_Vi              : in   std_logic;
		en_Vi_dec          : in   std_logic;
		en_L               : in   std_logic;
		en_delta1          : in   std_logic;
		en_delta0          : in   std_logic;
        en_U             : in   std_logic;
        en_input_tag     : in   std_logic;
        en_sum           : in   std_logic;		  
		en_S               : in   std_logic;
		en_Enc_Dec         : in   std_logic;
		rst_sum            : in   std_logic;
		bdo_valid		   : in  std_logic;
		start_len_cal	   : in  std_logic;
		decrypt			   : in  std_logic;
		bdo_size		   : out std_logic_vector(4 downto 0)
	  );


end copa_datapath;
    
architecture basic_arc of copa_datapath is

    signal EK1              : std_logic_vector(127 downto 0);
	 signal EK1inv           : std_logic_vector(127 downto 0);
	 signal Vi               : std_logic_vector(127 downto 0);
	 signal Vi_1             : std_logic_vector(127 downto 0);
	 signal Ci               : std_logic_vector(127 downto 0);
	 signal Mi               : std_logic_vector(127 downto 0);
	 signal Tag              : std_logic_vector(127 downto 0);
	 signal tag1             : std_logic_vector(127 downto 0);
	 signal tag2             : std_logic_vector(127 downto 0);
	 signal pmac             : std_logic_vector(127 downto 0);
	 signal mux1_out         : std_logic_vector(127 downto 0);
	 signal mux2_out         : std_logic_vector(127 downto 0);
	 signal Enc_Dec          : std_logic_vector(127 downto 0);
	 signal Enc_Dec2         : std_logic_vector(127 downto 0);
	 signal muxVi_out        : std_logic_vector(127 downto 0);
	 signal mux_Vi_dec_out   : std_logic_vector(127 downto 0);
	 signal muxdelta_out     : std_logic_vector(127 downto 0);
	 signal muxCi_out        : std_logic_vector(127 downto 0);
	 signal muxMi_out        : std_logic_vector(127 downto 0);
	 signal muxU_out         : std_logic_vector(127 downto 0);
	 signal muxsum_out       : std_logic_vector(127 downto 0);
	 signal S                : std_logic_vector(127 downto 0);
	 signal Vi_1_dec         : std_logic_vector(127 downto 0);
	 signal L                : std_logic_vector(127 downto 0);
	 signal L3               : std_logic_vector(127 downto 0);
	 signal L3x3             : std_logic_vector(127 downto 0);
	 signal Lp1              : std_logic_vector(127 downto 0);
	 signal Lp0              : std_logic_vector(127 downto 0);
	 signal Lp2              : std_logic_vector(127 downto 0);
	 signal Lp3              : std_logic_vector(127 downto 0);
	 signal Lp3x3            : std_logic_vector(127 downto 0);
	 signal Lp3x3x3          : std_logic_vector(127 downto 0);
	 signal z1               : std_logic_vector(127 downto 0);
	 signal z2               : std_logic_vector(127 downto 0);
	 signal U                : std_logic_vector(127 downto 0);
	 signal Up               : std_logic_vector(127 downto 0);
	 signal V1               : std_logic_vector(127 downto 0);
	 signal sum              : std_logic_vector(127 downto 0);
	 signal sum_reg          : std_logic_vector(127 downto 0);
	 signal f3               : std_logic_vector(127 downto 0);
	 signal f7               : std_logic_vector(127 downto 0);
	 signal reg_Enc_Dec_out  : std_logic_vector(127 downto 0);
	 signal s_in             : std_logic_vector(127 downto 0);
	 signal done_init1       : std_logic;
	 signal done_init2       : std_logic;

    signal Ci_p         	 : std_logic_vector(127 downto 0);		 
	 --v2-----------------------
	 signal f3x7     	     : std_logic_vector(127 downto 0);
	 signal f_out    	     : std_logic_vector(127 downto 0);
	 signal f7x2             : std_logic_vector(127 downto 0);
	 signal padded_Mi        : std_logic_vector(127 downto 0);
	 signal pad_loc_sig      : std_logic_vector(127 downto 0);
	 signal valid_bytes_sig  : std_logic_vector(127 downto 0);
	 signal Lp3x7		     : std_logic_vector(127 downto 0);
	 signal mux_Lp_out		 : std_logic_vector(127 downto 0);
	 signal bdo_size_dec     : std_logic_vector(4 downto 0);

begin


-------------------------------------------------------
--AES Core1
-------------------------------------------------------    
	 with init_in1 select
	    mux1_out <= (others => '0') when "000", 
		              EK1            when "001",
						  EK1inv         when "010",
						  tag1           when "011",
						  tag2           when "100",
						  pmac           when "101",
						  V1             when others;

------------------------------------------------------						  
	 AES1: entity work.AES_EncDec(structure)
    generic map( G_RNDS => 10)	 
    port map (
             clk        => clk,
             rst        => rst,
				 din        => mux1_out,
				 key        => key,
				 dout       => Enc_Dec,
				 init       => init1,
				 done_init  => done_init1,
				 decrypt    => init_Enc_Dec,
				 start      => start1,
				 ready      => ready1,
				 done       => done1
				 );
-------------------------------------------------------
--AES Core2
-------------------------------------------------------
    mux2_out <= Vi when init_Enc_Dec='0' else (Vi_1_dec xor Enc_Dec);
-------------------------------------------------------		
	 AES2: entity work.AES_EncDec(structure)
    generic map( G_RNDS => 10)	 
    port map (
             clk        => clk,
             rst        => rst,
				 din        => mux2_out,
				 key        => key,
				 dout       => Enc_Dec2,
				 init       => init2,
				 done_init  => done_init2,
				 decrypt    => init_Enc_Dec,
				 start      => start2,
				 ready      => ready2,
				 done       => done2
				 );
-------------------------------------------------------
     done_init_both <= done_init1 and done_init2;
-------------------------------------------------------
            
    muxVi_out <= (L xor Enc_Dec) when init_vi='1' else Vi;
	 
	 
	 reg128_Vi: entity work.Register_s(reg_arc) 
    port map (
             d        => muxVi_out,
             reset    => rst,
				 clock    => clk,
				 enable   => en_Vi,
				 q        => Vi_1
				 );
	 Vi <= Enc_Dec xor Vi_1;
	 
	 s_in <= Vi_1_dec when init_enc_dec_tag = '1' else Vi;
	 
	 reg128_S: entity work.Register_s(reg_arc) 
    port map (
             d        => s_in,
             reset    => rst,
				 clock    => clk,
				 enable   => en_S,
				 q        => S
				 );
				 
	 reg128_L: entity work.Register_s(reg_arc) 
    port map (
             d        => Enc_Dec,
             reset    => rst,
				 clock    => clk,
				 enable   => en_L,
				 q        => L
				 );
				 
	  mux_vi_dec_out <= (L xor Enc_Dec) when init_vi='1' else reg_Enc_Dec_out;			 
		
     reg128_Vi_dec: entity work.Register_s(reg_arc) 
     port map (
                d        => mux_vi_dec_out,
                reset    => rst,
	    		    clock    => clk,
				    enable   => en_Vi_dec,
				    q        => Vi_1_dec
				  );				  

     reg128_Enc_Dec: entity work.Register_s(reg_arc) 
     port map (
                d        => Enc_Dec,
                reset    => rst,
	    		    clock    => clk,
				    enable   => en_Enc_Dec,
				    q        => reg_Enc_Dec_out
				  );						  
--------------------------------------------------------	
--Encrypt-Decrypt
-------------------------------------------------------- 
     multi3_L: entity work.Multi_3(Multi_3_arc) 
     port map (
                input     => L,
                output    => L3
					 );
					 
	   multi3_Lx3: entity work.Multi_3(Multi_3_arc) 
      port map (
                input     => L3,
                output    => L3x3
					 );			 
					 
	  muxdelta_out <= L when init_pmac='0' else L3x3;

     Delta1: entity work.Delta(Delta_arc) 
     port map (
                input              => muxdelta_out,
			    init_delta         => init_delta,
			    rst        		   => rst,
                clk                => clk,
				en                 => en_delta1,
                output             => Lp1
					 );
      multi2_delta1: entity work.Multi_2(Multi_2_arc) 
      port map (
                 input     => Lp1,
                 output    => Lp2
					 );
		
      Delta0: entity work.Delta(Delta_arc) 
       port map (
                input              => muxdelta_out,
				init_delta         => init_delta,
				rst        		   => rst,
                clk                => clk,
				en                 => en_delta0,
                output             => Lp0
					 );
      multi3_delta0: entity work.Multi_3(Multi_3_arc) 
       port map (
                 input     => Lp0,
                 output    => Lp3
					 );		
	  multi3X7_delta0: entity work.Multi_7(Multi_7_arc) 
       port map (
                 input     => Lp3,
                 output    => Lp3X7
					 );
					 
		mux_Lp_out <= Lp3 when init_partial_delta ='0' else Lp3x7;
					 
---------------------------------------------------------	  
      muxCi_out <= Enc_Dec2 when init_Enc_Dec='0' else bdi;
      EK1inv    <= Lp2 xor muxCi_out;
      Ci        <= EK1inv;

      muxMi_out <= Enc_Dec2 when init_Enc_Dec='1' else bdi;
      EK1       <= mux_Lp_out xor muxMi_out;
      Mi        <= EK1;
----------------------------------------------------------
--PMAC1
----------------------------------------------------------
      pmac <= bdi xor Lp3; 

      multi3_pmac1: entity work.Multi_3(Multi_3_arc) 
       port map (
                input     => Lp3,
                output    => Lp3x3
					 );		
		multi3_pmac2: entity work.Multi_3(Multi_3_arc) 
       port map (
                input     => Lp3x3,
                output    => Lp3x3x3
					 );
         
		 Z1 <= Lp3x3x3 when init_size_eq='1' else Lp3x3;
-----------------------------------------------------------
		 muxU_out <= (others =>'0') when init_U='1' else Up;
			
		 reg128_U: entity work.Register_s(reg_arc) 
       port map (
                 d        => muxU_out,
                 reset    => rst,
				     clock    => clk,
				     enable   => en_U,
				     q        => U
				     );
					  
		 Up <= U xor Enc_Dec;
--------------------------------------------------------------
       V1 <= U xor Z1 xor bdi;		 
---------------------------------------------------------------
--SUM
---------------------------------------------------------------
		 padded_Mi  <= (Mi and valid_bytes_sig) or pad_loc_sig;  
		 muxsum_out <= bdi when init_enc_dec_tag='0' else padded_Mi; 
		 sum_reg    <= muxsum_out xor sum;
		 
		 reg128_sum: entity work.Register_s(reg_arc) 
         port map (
                   d        => sum_reg,
                   reset    => rst_sum,
				       clock    => clk,
				       enable   => en_sum,
				       q        => sum
				       );
----------------------------------------------------------------
--TAG
----------------------------------------------------------------
       multi3_f: entity work.Multi_3(Multi_3_arc) 
       port map (
                input     => mux_Lp_out,
                output    => f3
					 );		
		 
		 tag1  <= sum xor f3;
		 tag2  <= Enc_Dec xor S;
		 
		 multi7_f: entity work.Multi_7(Multi_7_arc) 
       port map (
                input     => Lp0,
                output    => f7
					 );
	   multi7_2_f: entity work.Multi_2(Multi_2_arc) 
       port map (
                input     => f7,
                output    => f7x2
					 );
					 
		 Tag <= Enc_Dec xor f7x2;
		 
-- TAG Compare
       tag_match <= '1' when bdi = Tag else '0' ; 
----------------------------------------------------------------
--OUTPUT
----------------------------------------------------------------
       Ci_p <= Ci   when init_tag ='0' else Tag;
       bdo  <= Ci_p when init_out ='0' else padded_Mi;

----------------------------------------------------------------
--BDO_SIZE CALCULATION
----------------------------------------------------------------
bdo_size_cal: entity work.length_cal(Behavioral) 
         port map (
                    clk         => clk,
                    rst         => rst,
				    start       => start_len_cal,
				    done   	    => open,
				    bdo_valid   => bdo_valid,
					input_block => Mi,
					bdo_size    => bdo_size_dec	
				    );

	bdo_size <= "10000" when decrypt ='0' else bdo_size_dec;

		 
----------------------------------------------------------------
--PADDING 
----------------------------------------------------------------		 
 valid_bytes : entity work.bit_expand (structure)
		port map ( input      => bdi_valid_bytes,
				   output     => valid_bytes_sig
					);
 pad_loc : entity work.bit_expand_pad_loc (structure)
		port map ( input      => bdi_pad_loc,
				   output     => pad_loc_sig
					);

      
        			
						  
end basic_arc;


