-------------------------------------------------------------------------------
--! @file       length_cal.vhd
--! @author     Farnoud Farahmand
--! @brief      Calculating the length of input
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity length_cal is
     port(  
            clk       : in  std_logic;
            rst       : in  std_logic;
		    start     : in  std_logic;
			done	  : out std_logic;
			bdo_valid : in  std_logic;
			input_block  : in   std_logic_vector(127 downto 0);
			bdo_size     : out  std_logic_vector(4 downto 0)
			);
			
end length_cal;

architecture Behavioral of length_cal is

      --! Internal Registers
      type t_state is (S_WAIT_START, S_CAL_LEN, S_CAL_DONE );
							 
							  
		signal state            : t_state;
      signal state_next       : t_state;
		signal en_reg           : std_logic;
		signal reg_output       : std_logic_vector(127 downto 0);
		signal reg_input        : std_logic_vector(127 downto 0);
		signal shifted_block    : std_logic_vector(127 downto 0);
		signal init_reg_input   : std_logic;
		signal len_reg          : std_logic_vector(4 downto 0);
		signal len_next         : std_logic_vector(4 downto 0);
		
		
begin
   
	 p_fsm: process(clk)
     begin
        if rising_edge(clk) then
            if (rst = '1') then
                state   <= S_WAIT_START;
            else
				state   <= state_next;
				len_reg <= len_next;
            end if;
		end if;	 
      end process;
		
		
    p_comb: process(state, start, reg_output, len_reg,  bdo_valid)
	    begin
          --! Default values
			state_next      <= state;
			len_next        <= len_reg;
         en_reg          <= '0';
          init_reg_input  <= '0';
			done		        <= '0';
			 	
		case state is
			  when S_WAIT_START =>
					len_next			<= "10000";
					if (start = '1')then
							init_reg_input  <= '1';
							en_reg 			<= '1';
							state_next      <= S_CAL_LEN;
					end if;		
							
				when S_CAL_LEN => 
					if (len_reg = "00000")then
							len_next	   <= "10000";
							state_next <= S_CAL_DONE;
					else
							if (reg_output(7 downto 0) = x"00")then
									len_next    <= len_reg - 1;
									en_reg 	   <= '1';
									state_next <= S_CAL_LEN;
							else
									if (reg_output(7 downto 0) = x"80")then
											len_next    <= len_reg -1;
											state_next <= S_CAL_DONE;
									else	
											len_next	   <= "10000";
											state_next <= S_CAL_DONE;
									end if;
							end if;
					end if;
					
				when S_CAL_DONE =>
					bdo_size <= len_reg;
					done 	 <= '1';
					if (bdo_valid = '1')then
							state_next <= S_WAIT_START;	
					end if;
				 
			end case;
		end process;
	
	right_shift: entity work.fixed_right_shift(dataflow)
                  GENERIC MAP (L => 8,
				               size => 128
				              )
                  PORT MAP ( a => reg_output ,
                             y => shifted_block
							 );
							 
	reg_input 	  <= shifted_block when init_reg_input = '0' else input_block;
	
    reg128: entity work.Register_s(reg_arc) 
        port map (
                d        => reg_input,
                reset    => rst,
				clock    => clk,
				enable   => en_reg,
				q        => reg_output
				);
				
end Behavioral;                               


